<style>
    #connections {
        width: 100%;
        margin-bottom: 0;
    }

    #connections th.ipport {
        font-size: 75%;
        font-weight: bold;
        width: 15%;
        white-space: nowrap;
    }
    #connections .time {
        width: 7rem;
        color: gray;
        font-size: 90%;
        padding-right: 1em;
        font-weight: normal;
    }

    #connections tfoot td {
        font-style: italic;
        color: gray;
    }

    #connections tr td, #connections tr th {
        vertical-align: top;
        line-height: 2.5rem;
    }

    #connections tr:first-child td, #connections tr:first-child th {
        border-top: none;
    }

    #connections tfoot th, #connections tfoot td {
        border: none;
    }

    .label {
        line-height: 2.5rem;
        vertical-align: middle;
    }

    td.uuid { width: 15rem; }

    .bg_red {
        background-color: #ffc0cf;
        color: black;
    }

    .bg_dred {
        background-color: red;
        color: yellow;
    }

    .bg_lred {
        background-color: #fee3e9;
        color: black;
    }

    .bg_lgreen {
        background-color: #defcda;
        color: black;
    }

    .bg_green {
        background-color: #bafba7;
        color: black;
    }

    .bg_dgreen {
        background-color: green;
        color: white;
    }

    .bg_yellow {
        background-color: #fefaae;
        color: black;
    }

    .bg_dyellow {
        background-color: yellow;
        color: black;
    }

    .bg_black {
        background-color: black;
        color: white;
    }

    .black {
        background-color: black;
        color: white;
    }

    .bg_white {
        background-color: #eee;
        color: black;
    }

    .bg_grey {
        background-color: grey;
        color: black;
    }

    .bg_lgrey {
        background-color: lightgrey;
        color: black;
    }
</style>
<script>

    function hit(uuid) {
        if (connections[uuid]) {
            clearTimeout(connections[uuid]);
        }

        connections[uuid] = setTimeout(function () {
            delete(connections[uuid]);
            recount();
        }, 10000);
        recount();
    }

    function recount() {
        $('#connections-count').text(Object.keys(connections).length);
    }

    $(document).ready(function () {
        recount();
        if ("WebSocket" in window) {
            wsConnect();
        }
    });

    function wsConnect() {
        const url = ((window.location.protocol.indexOf('s') !== -1) ? 'wss://' : 'ws://') +
            window.location.hostname +
            (window.location.port ? (':' + window.location.port) : '') +
            '/admin/haraka/11380/ws';
        ws = new WebSocket(url);

        ws.onopen = function () {
            $('#status').html("<i class=\"fa fa-circle fa-fw\" aria-hidden=\"true\"></i> {{ 'dashboard.realtime.connected'|trans }}").addClass('green');
        };
        ws.onerror = function (err) {
            console.log(err);
        };
        ws.onclose = function () {
            $('#status').html("<i class=\"fa fa-pause fa-fw\" aria-hidden=\"true\"></i> {{ 'dashboard.realtime.disconnected'|trans }}").removeClass('green');
            reconnect();
        };

        ws.onmessage = function (event) {
            const table = $('#connections tbody');

            const data = JSON.parse(event.data);

            console.log(data);
            if (data.uuid === undefined) return; // we will log only connections right now
            hit(data.uuid);

            const css_valid_uuid = get_css_safe_uuid(data.uuid);
            const css_valid_connection_uuid = get_css_safe_connection_id(data.uuid);
            const selector = 'tr#' + css_valid_uuid;

            if (!$(selector).length) {
                table.prepend('<tr id="' + css_valid_uuid + '" class="connection ' + css_valid_connection_uuid + '"><th class="time">' + getTime() + '</th><td class="uuid"><span class="label label-info">' + data.uuid.substr(16) + '...</span></td><th class="ipport"><span class="ip"></span><span class="port"></span></th><td class="labels"></td></tr>');
                $('#waiting').hide();
            }


            if (table.find('tr').length > 30) {
                table.find('tr:last').remove();
            }

            processData(selector, data, data.uuid);
        };
    }

    function getTime() {
        var now     = new Date();
        var hour    = now.getHours();
        var minute  = now.getMinutes();
        var second  = now.getSeconds();

        if(hour.toString().length === 1) {
            hour = '0'+hour;
        }
        if(minute.toString().length === 1) {
            minute = '0'+minute;
        }
        if(second.toString().length === 1) {
            second = '0'+second;
        }
        return hour+':'+minute+':'+second;
    }


    function reconnect() {
        setTimeout(function () {
            wsConnect();
        }, 1000);
    }

    function processData(selector, data, uuid) {
        for (var prop in data) {
            if (prop === 'uuid') continue;
            if (prop === 'queue' && data[prop]['title'] === undefined) continue;
            if (prop === 'dovecot_quota') continue; // buggy from some reason
            if (prop === 'max_unrecognized_commands') continue;
            if (prop === 'remote_host') {
                $(selector + ' .ip').text(data[prop]['title'] + ' → ');
                continue;
            }

            if (prop === 'local_port' && data[prop]['title'] !== 'disconnected') {
                $(selector + ' .port').text(data[prop]['newval']);
                continue;
            }

            applyLabel(selector, prop, data[prop]);
        }
    }

    function applyLabel(selector, plugin, plugin_data) {
        const classy = plugin_data.classy || 'bg_white';
        const label_selector = selector + ' .label.' + plugin;

        var title = plugin_data.title ? plugin + ': ' + plugin_data.title : plugin;
        if (plugin === 'rspamd') {
            title = plugin + ': ' + JSON.parse(plugin_data.title).score;
        } else if (plugin === 'tls') {
            title = plugin + ': ' + JSON.parse(plugin_data.title).cipher.name;
        } else if (plugin === 'dkim') {
            title = plugin;
        } else if (plugin === 'queue') {
            title = plugin_data.title;
        }

        if (!$(label_selector).length) {
            $(selector + ' td.labels').append('<span class="label ' + classy + ' ' + plugin + '"></span> ');
        } else {
            $(label_selector).removeClass().addClass("label " + classy + ' ' + plugin);
        }
        $(label_selector).text(title);
    }

    function get_css_safe_uuid(uuid) {
        const bits = uuid.split('.');
        if (bits.length === 1) {
            bits[1] = 1;
        }

        return 'aa_' + bits[0].replace(/[_-]/g, '') + '_' + bits[1];
    }

    function get_css_safe_connection_id(uuid) {
        const bits = uuid.split('.');

        return bits[0];
    }
</script>